require('Furry/fur');

-- Main
CharacterCreationMain.outfitSpeciesMappingFile = "FurryMod_saved_outfits_species.txt";

local originalMainLoadOutfit = CharacterCreationMain.loadOutfit;
local originalMainSaveBuildStep2 = CharacterCreationMain.saveBuildStep2;
local originalMainDeleteBuildStep2 = CharacterCreationMain.deleteBuildStep2;

OUTFIT_SPECIES_MAPPING_FILE_VERSION = 1;

function CharacterCreationMain.readOutfitSpeciesMappingFile()
    local retVal = {};

    local saveFile = getFileReader(CharacterCreationMain.outfitSpeciesMappingFile, true);
    local version = 0;
    local line = saveFile:readLine();
    while line ~= nil do
        if luautils.stringStarts(line, "VERSION=") then
            version = tonumber(string.split(line, "=")[2])
        elseif version == OUTFIT_SPECIES_MAPPING_FILE_VERSION then
            local s = luautils.split(line, ":");
            retVal[s[1]] = s[2];
        end
        line = saveFile:readLine();
    end
    saveFile:close();

    return retVal;
end

function CharacterCreationMain.writeOutfitSpeciesMappingsFile(outfitSpeciesMappings)
    local saveFile = getFileWriter(CharacterCreationMain.outfitSpeciesMappingFile, true, false); -- overwrite
    saveFile:write("VERSION="..tostring(OUTFIT_SPECIES_MAPPING_FILE_VERSION).."\n")
    for key,val in pairs(outfitSpeciesMappings) do
        saveFile:write(key..":"..val.."\n");
    end
    saveFile:close();
end

function CharacterCreationMain:updateButtonsForSpeciesGroup(groupName)
    local main = CharacterCreationMain.instance;
    if groupName == 'Hooman' then
        main.characterPanel:setVisible(true);
    else
        main.characterPanel:setVisible(false);
    end;
end

function CharacterCreationMain.loadOutfit(self, box)
    local outfitName = box.options[box.selected];
    local result = originalMainLoadOutfit(self, box);
    if outfitName == nil then return result end;

    local speciesGroupName = 'Hooman';
    local speciesName = 'Hooman';
    local outfitSpeciesMappings = CharacterCreationMain.readOutfitSpeciesMappingFile();
    local outfitSpecies = outfitSpeciesMappings[outfitName];
    if outfitSpecies and outfitSpecies ~= 'Hooman' then
        local speciesGroupForOutfitSpecies = FurManager.getSpeciesGroupForSpecies(outfitSpecies);
        if speciesGroupForOutfitSpecies then
            speciesGroupName = speciesGroupForOutfitSpecies.name;
            speciesName = outfitSpecies;
        else
            speciesGroupName = 'Hooman';
            speciesName = 'Hooman';
        end
    end

    MainScreen.instance.charCreationHeader:changeSpecies(speciesGroupName, speciesName);

    return result;
end

function CharacterCreationMain.saveBuildStep2(self, button, joypadData, param2)
    local outfitName = button.parent.entry:getText();
    local result = originalMainSaveBuildStep2(self, button, joypadData, param2);
    if outfitName == '' then return result end;

    local outfitFurMappings = CharacterCreationMain.readOutfitSpeciesMappingFile();
    outfitFurMappings[outfitName] = MainScreen.instance.charCreationHeader.speciesName;
    CharacterCreationMain.writeOutfitSpeciesMappingsFile(outfitFurMappings);

    return result;
end

function CharacterCreationMain.deleteBuildStep2(self, button, joypadData)
    local deletedOutfit = self.savedBuilds.options[self.savedBuilds.selected];
    local result = originalMainDeleteBuildStep2(self, button, joypadData);
    if deletedOutfit == '' then return result end;

    local outfitFurMappings = CharacterCreationMain.readOutfitSpeciesMappingFile();
    local newOutfitFurMappings = {};
    for outfit, species in pairs(outfitFurMappings) do
        if outfit ~= deletedOutfit then
            newOutfitFurMappings[outfit] = species;
        end
    end
    CharacterCreationMain.writeOutfitSpeciesMappingsFile(outfitFurMappings);

    return result;
end

-- Header

local originalHeaderNew = CharacterCreationHeader.new;
local originalHeaderCreate = CharacterCreationHeader.create;
local originalHeaderDressWithDefinitions = CharacterCreationHeader.dressWithDefinitions;
local originalHeaderOnGenderSelected = CharacterCreationHeader.onGenderSelected;

function CharacterCreationHeader:updatePreviewAfterSpeciesChanged()
    local desc = MainScreen.instance.desc;
    local furClothingName = FurManager.determineFurClothingName(desc:isFemale(), self.speciesName);
    local furClothing = InventoryItemFactory.CreateItem(furClothingName);
    desc:setWornItem(furClothing:getBodyLocation(), nil);
    desc:setWornItem(furClothing:getBodyLocation(), furClothing);
    self:setAvatarFromUI();
end;

function CharacterCreationHeader:changeSpecies(speciesGroupName, speciesName)
    self.speciesGroupName = speciesGroupName;
    self.speciesName = speciesName;
    self.speciesGroupCombo:selectData(self.speciesGroupName);
    if self.speciesGroupName == 'Hooman' then
        self.speciesCombo:setVisible(false);
    else
        self.speciesCombo:clear();
        FurManager.populateComboWithSpecies(self.speciesCombo, self.speciesGroupName);
        self.speciesCombo:selectData(self.species);
        self.speciesCombo:setVisible(true);
    end
    self:updatePreviewAfterSpeciesChanged();
    CharacterCreationMain.instance:updateButtonsForSpeciesGroup(self.speciesGroupName);
    CharacterCreationMain.instance:loadJoypadButtons();
end

function CharacterCreationHeader:onSpeciesComboChange(combo)
    local speciesName = combo:getOptionData(combo.selected);
    self:changeSpecies(self.speciesGroupName, speciesName);
end;

function CharacterCreationHeader:onSpeciesGroupComboChange(combo)
    local groupName = combo:getOptionData(combo.selected);
    if groupName == 'Hooman' then
        self:changeSpecies('Hooman', 'Hooman');
    else
        local group = FurManager.getSpeciesGroup(groupName);
        self:changeSpecies(groupName, group.species[1].name);
    end
end;

function CharacterCreationHeader.onGenderSelected(self, combo)
    originalHeaderOnGenderSelected(self, combo);

    CharacterCreationMain:updateButtonsForSpeciesGroup(self.speciesGroupName);
end

function CharacterCreationHeader.dressWithDefinitions(self, definition, resetWornItems)
    originalHeaderDressWithDefinitions(self, definition, resetWornItems);

    self:updatePreviewAfterSpeciesChanged();
end

function CharacterCreationHeader.create(self)
    originalHeaderCreate(self);

    self.genderCombo:setX(self.avatarPanel:getRight() + 16);

    local speciesGroupComboX = self.genderCombo:getRight() + 6;
    local speciesGroupComboY = self.genderCombo.y;
    local speciesGroupComboWidth = self.genderCombo.width + 6;
    local speciesGroupComboHeight = self.genderCombo.height;
    self.speciesGroupCombo = ISComboBox:new(speciesGroupComboX, speciesGroupComboY, speciesGroupComboWidth, speciesGroupComboHeight, self, CharacterCreationHeader.onSpeciesGroupComboChange);
    self.speciesGroupCombo:initialise();
    FurManager.populateComboWithSpeciesGroups(self.speciesGroupCombo);
    self.speciesGroupCombo.borderColor = { r=0.4, g=0.4, b=0.4, a=1};
    self:addChild(self.speciesGroupCombo);

    local speciesComboX = self.speciesGroupCombo:getRight() + 6;
    local speciesComboY = self.speciesGroupCombo.y;
    local speciesComboWidth = self.speciesGroupCombo.width + 6;
    local speciesComboHeight = self.speciesGroupCombo.height;
    self.speciesCombo = ISComboBox:new(speciesComboX, speciesComboY, speciesComboWidth, speciesComboHeight, self, CharacterCreationHeader.onSpeciesComboChange);
    self.speciesCombo:initialise();
    self.speciesCombo.borderColor = { r=0.4, g=0.4, b=0.4, a=1};
    self:addChild(self.speciesCombo);
    self.speciesCombo:setVisible(false);
end

function CharacterCreationHeader.new(self, x, y, width, height)
    local o = originalHeaderNew(self, x, y, width, height);
    o.speciesGroupName = "Hooman";
    o.speciesName = "Hooman";
    return o;
end