FurManager = {}

FurManager.speciesGroups = {};

local function clothingToBodyBloodType(clothingBloodType)
    local name = tostring(clothingBloodType);

    for i=1,BloodBodyPartType.MAX:index() do
        local part = BloodBodyPartType.FromIndex(i-1)
        if name == tostring(part) then return part end
    end

    return nil;
end

function FurManager.registerSpeciesGroup(speciesGroup)
    if not speciesGroup.name then
        print("Missing name when registering species group");
        return;
    end

    print('Registering species group: ', speciesGroup.name);

    if not speciesGroup.display then
        print("Missing display text when registering species group");
        return;
    end

    if not speciesGroup.species then
        print("Missing species array when registering species group");
        return;
    end

    for i,species in ipairs(speciesGroup.species) do
        if not species.name then
            print("Missing name on species when registering species group ", speciesGroup.name)
            return;
        end

        print('Registering species: ', speciesGroup.name);

        if not species.display then
            print("Missing display on species ", species.name, " when registering species group ", speciesGroup.name)
            return;
        end

        if not species.maleClothing then
            print("Missing maleClothing on species ", species.name, " when registering species group ", speciesGroup.name)
            return;
        end

        if not species.femaleClothing then
            print("Missing femaleClothing on species ", species.name, " when registering species group ", speciesGroup.name)
            return;
        end
    end

    table.insert(FurManager.speciesGroups, speciesGroup);
end

function FurManager.getSpeciesGroup(groupName)
    for i, group in ipairs(FurManager.speciesGroups) do
        if group.name == groupName then
            return group;
        end
    end;

    return nil;
end

function FurManager.getSpeciesGroupForSpecies(speciesName)
    for i, group in ipairs(FurManager.speciesGroups) do
        for j, species in ipairs(group.species) do
            if species.name == speciesName then
                return group;
            end
        end
    end;

    return nil;

end

function FurManager.populateComboWithSpeciesGroups(comboBox)
    comboBox:addOptionWithData("Human", "Hooman");
    for i, group in ipairs(FurManager.speciesGroups) do
        comboBox:addOptionWithData(group.display, group.name);
    end
end

function FurManager.populateComboWithSpecies(comboBox, groupName)
    local group = FurManager.getSpeciesGroup(groupName);
    if group ~= nil then
        for j, species in ipairs(group.species) do
            comboBox:addOptionWithData(species.display, species.name);
        end
    end
end

function FurManager.createSpeciesCombo(parent, x, y, width, height, callback)
    local comboBox = ISComboBox:new(x, y, width, height, parent, callback);
    comboBox:initialise();
    comboBox:addOptionWithData("Human", "Hooman");
    for i, group in ipairs(FurManager.speciesGroups) do
        for j, species in ipairs(group.species) do
            comboBox:addOptionWithData(species.display, species.name);
        end
    end
    return comboBox;
end

function FurManager.getAllFurClothingNames(isFemale)
    local result = {};
    table.insert(result, "Base.Furry_HoomanPlaceholder");
    for i,speciesGroup in ipairs(FurManager.speciesGroups) do
        for j,species in ipairs(speciesGroup.species) do
            if isFemale then
                table.insert(result, species.femaleClothing);
            else
                table.insert(result, species.maleClothing);
            end
        end
    end
    return result;
end

function FurManager.determineFurClothingName(isFemale, speciesName)
    if speciesName == "Hooman" then return "Base.Furry_HoomanPlaceholder" end;

    for i,speciesGroup in ipairs(FurManager.speciesGroups) do
        for j,species in ipairs(speciesGroup.species) do
            if species.name == speciesName then
                if isFemale then
                    print('Species clothing determined: ', species.femaleClothing)
                    return species.femaleClothing
                else
                    print('Species clothing determined: ', species.maleClothing)
                    return species.maleClothing
                end
            end
        end
    end

    print('Unknown species requested, returning fox: ', speciesName);
    if isFemale then
        return 'Base.Furry_FemaleFox';
    else
        return 'Base.Furry_MaleFox';
    end
end

function FurManager.isHoomanSkin(furClothing)
    return furClothing:getType() == "Furry_HoomanPlaceholder";
end

function FurManager.hasConfiguredMod(player)
   return FurManager.getPlayerInventoryFur(player) ~= nil;
end

function FurManager.removePlayerFur(player)
    local found = {};
    for i=0, player:getInventory():getItems():size()-1 do
        local item = player:getInventory():getItems():get(i);
        if item:hasTag("Fur") then
            table.insert(found, item);
        end
    end;

    for i, inventoryItem in ipairs(found) do
        inventoryItem:setFavorite(false);
        player:setWornItem(inventoryItem:getBodyLocation(), nil);
        player:getInventory():Remove(inventoryItem);
    end

    player:resetModel();
    sendClothing(player);
end

function FurManager.getPlayerWornFur(player)
    for i=0, player:getWornItems():size()-1 do
        local item = player:getWornItems():getItemByIndex(i);
        if item:hasTag("Fur") then return item end
    end;

    return nil;
end;

function FurManager.getPlayerInventoryFur(player)
    for i=0, player:getInventory():getItems():size()-1 do
        local item = player:getInventory():getItems():get(i);
        if item:hasTag("Fur") then return item end
    end;

    return nil;
end;

function FurManager.forceFurWornState(player)
    local inventoryItem = FurManager.getPlayerInventoryFur(player);
    if inventoryItem == nil then return end

    inventoryItem:setFavorite(true);

    local wornItem = FurManager.getPlayerWornFur(player);
    if wornItem == nil and not FurManager.isHoomanSkin(inventoryItem) then
        print('Re-equipping fur that was removed.');
        player:setWornItem(inventoryItem:getBodyLocation(), inventoryItem);
        player:resetModel();
        sendClothing(player);
    elseif wornItem ~= nil and FurManager.isHoomanSkin(inventoryItem) then
        print('Un-equipping skin that was worn.')
        wornItem:setFavorite(true);
        player:setWornItem(inventoryItem:getBodyLocation(), nil);
    end
end

function FurManager.syncFurWetness(player)
    local furClothing = FurManager.getPlayerWornFur(player);
    if furClothing == nil then return end;

    local bodyWetness = player:getBodyDamage():getWetness();
    local furWetness = furClothing:getWetness();
    if furWetness ~= bodyWetness then
        local highestWetness = Math.max(bodyWetness, furWetness);
        player:getBodyDamage():setWetness(highestWetness);
    end
end

function FurManager.syncFurCondition(player)
    local furClothing = FurManager.getPlayerWornFur(player);
    if furClothing == nil then return end;

    local bodySynced = false;

    local coveredParts = BloodClothingType.getCoveredParts(furClothing:getBloodClothingType());
    if coveredParts then
        for j = 0, coveredParts:size() - 1 do
            local clothingPart = coveredParts:get(j);
            local bodyPart = clothingToBodyBloodType(clothingPart);

            -- blood
            local clothingBlood = furClothing:getBlood(clothingPart);
            local bodyBlood = player:getVisual():getBlood(bodyPart);

            if bodyBlood ~= clothingBlood then
                player:getVisual():setBlood(bodyPart, clothingBlood);
                bodySynced = true;
            end

            -- dirt
            local clothingDirt = furClothing:getDirt(clothingPart);
            local bodyDirt = player:getVisual():getDirt(bodyPart);

            if bodyDirt ~= clothingDirt then
                player:getVisual():setDirt(bodyPart, clothingDirt);
                bodySynced = true;
            end
        end
    end;

    if (bodySynced) then
        player:resetModelNextFrame();
        sendVisual(player);
        triggerEvent("OnClothingUpdated", player);
    end
end;

function FurManager.swapFurToDeceased(player)
    local furClothing = FurManager.getPlayerInventoryFur(player);
    if furClothing == nil then return end;
    if FurManager.isHoomanSkin(furClothing) then return end;

    local furType = furClothing:getType();
    FurManager.removePlayerFur(player);

    local deceasedFurClothing = player:getInventory():AddItem(furType .. '_Deceased');
    player:setWornItem(deceasedFurClothing:getBodyLocation(), deceasedFurClothing);
    player:resetModel();
    sendClothing(player);
end;

function FurManager.removedDeceasedFur(player)
    local found = {};
    for i=0, player:getInventory():getItems():size()-1 do
        local item = player:getInventory():getItems():get(i);
        if item:hasTag("DeceasedFur") then
            table.insert(found, item);
        end
    end;

    for i, inventoryItem in ipairs(found) do
        inventoryItem:setFavorite(false);
        player:getInventory():Remove(inventoryItem);
    end
end

function FurManager.onPlayerDead(player)
    FurManager.swapFurToDeceased(player);
end

function FurManager.onGameBoot()
    local humanGroup = BodyLocations.getGroup("Human");
    local furLocation = humanGroup:getOrCreateLocation("Fur");
    local list = getClassFieldVal(humanGroup, getClassField(humanGroup, 1));
    list:remove(furLocation)
    local index = humanGroup:indexOf("Wound");
    list:add(index, furLocation);
end;

function FurManager.onCreatePlayer(playerNum)
    local player = getSpecificPlayer(playerNum);
    FurManager.forceFurWornState(player);
    FurManager.removedDeceasedFur(player); -- In case people end up having it for some reason
end

function FurManager.onPlayerUpdate(player)
    FurManager.syncFurWetness(player);
    FurManager.syncFurCondition(player);
end

function FurManager.onClothingUpdated(player)
    FurManager.forceFurWornState(player);
end

Events.OnGameBoot.Add(FurManager.onGameBoot)
Events.OnCreatePlayer.Add(FurManager.onCreatePlayer);
Events.OnPlayerDeath.Add(FurManager.onPlayerDead);
Events.OnPlayerUpdate.Add(FurManager.onPlayerUpdate);
Events.OnClothingUpdated.Add(FurManager.onClothingUpdated);