FurryShakeAction = ISBaseTimedAction:derive("FurryShakeAction");

function FurryShakeAction.isValidForPlayer(player)
    if player:getVehicle() then
        print("Shaking not valid: in vehicle");
        return false;
    end;

    local fur = FurManager.getPlayerWornFur(player);
    if fur == nil then
        print("Shaking not valid: no fur");
        return false;
    end;

    if player:getBodyDamage():getWetness() == 0 then
        print("Shaking not valid: fur not wet");
        return false;
    end;

    return true;
end

function FurryShakeAction:isValid()
    -- We only check when populating.
    return true;
end

function FurryShakeAction:update()
    self.tick = self.tick + 1;
    if self.tick >= self.timer then
        local fur = FurManager.getPlayerWornFur(self.character);
        local delta = fur:getWetness() / 20.0;
        local newWetness = fur:getWetness() - delta;
        self.tick = 0;
        fur:setWetness(newWetness);
        self.character:getBodyDamage():setWetness(newWetness);
    end
    self.character:setMetabolicTarget(Metabolics.LightDomestic);
end

function FurryShakeAction:start()
    print("Shake started!");
    self:setActionAnim("Furry_ShakeDry");
end

function FurryShakeAction:stop()
    print("Shake stopped!");
    triggerEvent("OnClothingUpdated", self.character);
    ISBaseTimedAction.stop(self);
end

function FurryShakeAction:perform()
    print("Shake performed!");
    triggerEvent("OnClothingUpdated", self.character);
    ISBaseTimedAction.perform(self);
end

function FurryShakeAction:new(character, time)
    local o = ISBaseTimedAction.new(self, character);
    o.fur = FurManager.getPlayerWornFur(character);
    o.stopOnAim = true;
    o.stopOnWalk = true;
    o.stopOnRun = true;
    o.maxTime = time;
    o.ignoreHandsWounds = true;
    o.animSpeed = 1.0;
    if o.character:isTimedActionInstant() then o.maxTime = 1; end
    o.timer = time / 20;
    o.tick = 0;
    o.caloriesModifier = 4;
    return o;
end

function FurryShakeAction.doesClothingNeedRemoval(clothing)
    if clothing:getBodyLocation() == "Hat" then return true end;
    if clothing:getBodyLocation() == "Mask" then return true end;
    if clothing:getBodyLocation() == "FullHat" then return true end;
    if clothing:getBodyLocation() == "MaskEyes" then return true end;
    if clothing:getBodyLocation() == "MaskFull" then return true end;
    if clothing:getBodyLocation() == "FullHelmet" then return true end;
    if clothing:getBodyLocation() == "Jacket" then return true end;
    if clothing:getBodyLocation() == "FullTop" then return true end;
    if clothing:getBodyLocation() == "JacketHat" then return true end;
    if clothing:getBodyLocation() == "Sweater" then return true end;
    if clothing:getBodyLocation() == "TorsoExtra" then return true end;
    if clothing:getBodyLocation() == "SweaterHat" then return true end;
    if clothing:getBodyLocation() == "Underwear" then return true end;
    if clothing:getBodyLocation() == "Tail" then return true end;
    if clothing:getBodyLocation() == "Legs1" then return true end;
    if clothing:getBodyLocation() == "Pants" then return true end;
    if clothing:getBodyLocation() == "Skirt" then return true end;
    if clothing:getBodyLocation() == "ShortSleeveShirt" then return true end;
    if clothing:getBodyLocation() == "Shirt" then return true end;
    if clothing:getBodyLocation() == "Tshirt" then return true end;
    if clothing:getBodyLocation() == "TankTop" then return true end;
    if clothing:getBodyLocation() == "Socks" then return true end;
    if clothing:getBodyLocation() == "UnderwearInner" then return true end;
    if clothing:getBodyLocation() == "Shoes" then return true end;
    if clothing:getBodyLocation() == "Torso1Legs1" then return true end;
    if clothing:getBodyLocation() == "BathRobe" then return true end;
    if clothing:getBodyLocation() == "FullSuit" then return true end;
    if clothing:getBodyLocation() == "FullSuitHead" then return true end;
    if clothing:getBodyLocation() == "Dress" then return true end;
    if clothing:getBodyLocation() == "UnderwearTop" then return true end;
    if clothing:getBodyLocation() == "UnderwearBottom" then return true end;
    if clothing:getBodyLocation() == "UnderwearExtra1" then return true end;
    if clothing:getBodyLocation() == "UnderwearExtra2" then return true end;
    return false;
end;

function FurryShakeAction.doShakeAction(player, square)
    local removedClothing = {};

    -- Remove clothing
    for i=0, player:getWornItems():size()-1 do
        local item = player:getWornItems():getItemByIndex(i);
        if FurryShakeAction.doesClothingNeedRemoval(item) then
            local unequipAction = ISUnequipAction:new(player, item, 50);
            ISTimedActionQueue.add(unequipAction);
            table.insert(removedClothing, item);
        end
    end;

    -- Shake dry
    local shakeAction = FurryShakeAction:new(player, 300);
    ISTimedActionQueue.add(shakeAction);

    -- Wear removed clothing
    for i, item in ipairs(removedClothing) do
        local wearAction = ISWearClothing:new(player, item, 50);
        ISTimedActionQueue.add(wearAction);
    end
end

function FurryShakeAction.onFillWorldObjectContextMenu(playerNum, context, worldObjects, test)
    if test then return end;

    local player = getSpecificPlayer(playerNum);

    if FurryShakeAction.isValidForPlayer(player) then
        context:addOption("Shake Dry", player, FurryShakeAction.doShakeAction, player:getSquare());
    end
end

Events.OnFillWorldObjectContextMenu.Add(FurryShakeAction.onFillWorldObjectContextMenu);