require "ISUI/ISButton"
require "ISUI/ISPanelJoypad"

FurrySpeciesSelectDialog = ISPanelJoypad:derive("FurrySpeciesSelectDialog");

local largeFontHeight = getTextManager():getFontHeight(UIFont.Large)
local smallFontHeight = getTextManager():getFontHeight(UIFont.Small)

function FurrySpeciesSelectDialog:initialise()
    ISPanelJoypad.initialise(self);

    local paddingX = 8;
    local paddingY = 8;

    local titleLabelX = paddingX;
    local titleLabelY = paddingY;
    local titleLabelText = "Hold onto your fur!";
    local titleLabelWidth = getTextManager():MeasureStringX(UIFont.Large, titleLabelText);
    local titleLabelHeight = largeFontHeight;
    local titleLabel = ISLabel:new(titleLabelWidth + titleLabelX, titleLabelY, titleLabelHeight, titleLabelText, 1, 1, 1, 1, UIFont.Large, false);
    titleLabel:initialise();
    titleLabel:instantiate();
    self:addChild(titleLabel);

    local messageLabelText;
    if SandboxVars.FurryMod.AllowChange then
        messageLabelText = "Please select a new species below.";
    else
        messageLabelText = "It looks like you started this game without Anthro Survivors.\nIf you want a new species, this is your one and only chance.";
    end

    local messageLabelWidth = getTextManager():MeasureStringX(UIFont.Small, messageLabelText);
    local messageLabelX = paddingX;
    local messageLabelY = titleLabelHeight + paddingY;
    local messageLabelHeight = smallFontHeight * 2;
    local messageLabel = ISLabel:new(messageLabelWidth + messageLabelX, messageLabelY, messageLabelHeight, messageLabelText, 1, 1, 1, 1, UIFont.Small, false);
    messageLabel:initialise();
    messageLabel:instantiate();
    self:addChild(messageLabel);

    local speciesLabelText = "Species: ";
    local speciesLabelWidth = getTextManager():MeasureStringX(UIFont.Small, speciesLabelText);
    local speciesLabelX = paddingX;
    local speciesLabelY = messageLabelY + titleLabelHeight + paddingY;
    local speciesLabelHeight = smallFontHeight;
    local speciesLabel = ISLabel:new(speciesLabelWidth + paddingX, speciesLabelY, speciesLabelHeight, speciesLabelText, 1, 1, 1, 1, UIFont.Small, false);
    speciesLabel:initialise();
    speciesLabel:instantiate();
    self:addChild(speciesLabel);

    local speciesGroupComboX = speciesLabelX + speciesLabelWidth + paddingX;
    local speciesGroupComboY = speciesLabel.y;
    local speciesGroupComboWidth = 160;
    local speciesGroupComboHeight = smallFontHeight + 6;
    self.speciesGroupCombo = ISComboBox:new(speciesGroupComboX, speciesGroupComboY, speciesGroupComboWidth, speciesGroupComboHeight, self, FurrySpeciesSelectDialog.onSpeciesGroupChanged);
    self.speciesGroupCombo:initialise();
    FurManager.populateComboWithSpeciesGroups(self.speciesGroupCombo);
    self.speciesGroupCombo.borderColor = {r=0.4, g=0.4, b=0.4, a=1};
    self:addChild(self.speciesGroupCombo);

    local speciesComboX = speciesGroupComboX;
    local speciesComboY = speciesGroupComboY + speciesGroupComboHeight;
    local speciesComboWidth = speciesGroupComboWidth;
    local speciesComboHeight = speciesGroupComboHeight;
    self.speciesCombo = ISComboBox:new(speciesComboX, speciesComboY, speciesComboWidth, speciesComboHeight, self, FurrySpeciesSelectDialog.onSpeciesChanged);
    self.speciesCombo:initialise();
    self.speciesCombo.borderColor = { r=0.4, g=0.4, b=0.4, a=1};
    self:addChild(self.speciesCombo);
    self.speciesCombo:setVisible(false);

    local confirmButtonWidth = getTextManager():MeasureStringX(UIFont.Small, "Confirm") + 8;
    local confirmButtonHeight = largeFontHeight;
    local confirmButtonX = paddingX;
    local confirmButtonY = speciesComboY + speciesComboHeight + paddingY;
    self.confirmButton = ISButton:new(confirmButtonX, confirmButtonY, confirmButtonWidth, confirmButtonHeight, "CONFIRM", self, FurrySpeciesSelectDialog.onConfirmed);
    self.confirmButton:initialise();
    self.confirmButton:instantiate();
    self:addChild(self.confirmButton);

    self:insertNewLineOfButtons(self.confirmButton);
    self:setHeight(confirmButtonY + confirmButtonHeight + paddingY);
end

function FurrySpeciesSelectDialog:destroy()
    UIManager.setShowPausedMessage(true);
    self:setVisible(false);
    self:removeFromUIManager();
    if UIManager.getSpeedControls() then
        UIManager.getSpeedControls():SetCurrentGameSpeed(1);
    end
end

function FurrySpeciesSelectDialog:onConfirmed()
    self.player:resetModel();
    sendClothing(self.player);
    self:destroy();
end

function FurrySpeciesSelectDialog:updatePlayer(species)
    FurManager.removePlayerFur(self.player);

    local furClothingName = FurManager.determineFurClothingName(self.player:isFemale(), species);
    local furClothing = self.player:getInventory():AddItem(furClothingName);
    furClothing:setFavorite(true);
    if not FurManager.isHoomanSkin(furClothing) then
        self.player:setWornItem(furClothing:getBodyLocation(), furClothing);
    end
    triggerEvent("OnClothingUpdated", self.player);
end

function FurrySpeciesSelectDialog:onSpeciesChanged(combo)
    local species = combo:getOptionData(combo.selected);
    self:updatePlayer(species);
end;

function FurrySpeciesSelectDialog:onSpeciesGroupChanged(combo)
    local groupName = combo:getOptionData(combo.selected);
    if groupName == 'Hooman' then
        self.speciesCombo:setVisible(false);
        self:updatePlayer('Hooman');
    else
        local group = FurManager.getSpeciesGroup(groupName);
        local species = group.species[1].name;
        self.speciesCombo:clear();
        FurManager.populateComboWithSpecies(self.speciesCombo, groupName);
        self.speciesCombo:selectData(species);
        self.speciesCombo:setVisible(true);
        self:updatePlayer(species);
    end
end

function FurrySpeciesSelectDialog:render()

end

function FurrySpeciesSelectDialog:update()
    ISPanelJoypad.update(self);

    if self.confirmButton.joypadFocused then
        self.confirmButton:setJoypadButton(Joypad.Texture.AButton);
    else
        self.confirmButton.joypadTexture = nil;
        self.confirmButton.isJoypad = false;
    end
end


function FurrySpeciesSelectDialog:onGainJoypadFocus(joypadData)
    ISPanelJoypad.onGainJoypadFocus(self, joypadData)
    self.joypadIndexY = 1
    self.joypadIndex = 1
    self.joypadButtons = self.joypadButtonsY[self.joypadIndexY]
    self.joypadButtons[self.joypadIndex]:setJoypadFocused(true)
end

function FurrySpeciesSelectDialog:onJoypadDown(button)
    ISPanelJoypad.onJoypadDown(self, button)
    if button == Joypad.BButton then
        self:onClick(self.confirmButton)
    end
end

function FurrySpeciesSelectDialog:new(x, y, width, height, playerNum)
    local player = getSpecificPlayer(playerNum);
    local o = {}
    o = ISPanelJoypad:new(x, y, width, height);
    setmetatable(o, self)
    self.__index = self
    o.backgroundColor = {r=0, g=0, b=0, a=0.5};
    o.borderColor = {r=0.4, g=0.4, b=0.4, a=1};
    if y == 0 then
        o.y = getPlayerScreenTop(playerNum) + (getPlayerScreenHeight(playerNum) - height) / 4;
        o:setY(o.y)
    end
    if x == 0 then
        o.x = getPlayerScreenLeft(playerNum) + (getPlayerScreenWidth(playerNum) - width) / 2;
        o:setX(o.x)
    end
    o.width = width;
    o.height = height;
    o.anchorLeft = true;
    o.anchorRight = true;
    o.anchorTop = true;
    o.anchorBottom = true;
    o.player = player;
    return o;
end

function FurrySpeciesSelectDialog.show(player)
    FurManager.removePlayerFur(player);
    local furClothingName = FurManager.determineFurClothingName(player:isFemale(), "Hooman");
    player:getInventory():AddItem(furClothingName);
    triggerEvent("OnClothingUpdated", player);

    local modal = FurrySpeciesSelectDialog:new(0, 0, 480, 320, player:getPlayerNum());
    modal:initialise();
    modal:addToUIManager();
end

function FurrySpeciesSelectDialog.onCreatePlayer(playerNum)
    if isServer() then return end;
    local player = getSpecificPlayer(playerNum);
    if FurManager.hasConfiguredMod(player) then return end;
    FurrySpeciesSelectDialog.show(player);
end

function FurrySpeciesSelectDialog.onChangeSpecies(player, square)
    if isServer() then return end;
    FurrySpeciesSelectDialog.show(player);
end

function FurrySpeciesSelectDialog.onFillWorldObjectContextMenu(playerNum, context, worldObjects, test)
    if test then return end;

    local player = getSpecificPlayer(playerNum);
    if SandboxVars.FurryMod.AllowChange then
        context:addOption("Change Species", player, FurrySpeciesSelectDialog.onChangeSpecies, player:getSquare());
    end
end

Events.OnCreatePlayer.Add(FurrySpeciesSelectDialog.onCreatePlayer);
Events.OnFillWorldObjectContextMenu.Add(FurrySpeciesSelectDialog.onFillWorldObjectContextMenu);
